"""For help on EMAN, look up libpyEM instead of EMAN"""
from libpyEM import *
#from libpyCylinder import *
import sys, os, os.path
import tempfile
import math
import time
import string

EMANVERSION="EMAN2 v1.9"

EMData.MRC = Image.mrc
EMData.SPIDER = Image.spider
EMData.IMAGIC = Image.img
EMData.PGM = Image.pgm
EMData.LST = Image.lst
EMData.PIF = Image.pif
EMData.PNG = Image.png
EMData.HDF = Image.hdf5
EMData.DM3 = Image.dm3
EMData.TIF = Image.tif
EMData.VTK = Image.vtk
EMData.SAL = Image.sal
EMData.ICOS = Image.map
EMData.IMAGIC3D = Image.img3d
EMData.EMIM = Image.emim
EMData.SINGLE_SPIDER = Image.single_spider
EMData.GATAN = Image.dm2
EMData.AMIRA = Image.amira
EMData.XPLOR = Image.xplor
EMData.ANY = Image.any

Euler.EMAN = EulerType.eman
Euler.IMAGIC = EulerType.imagic
Euler.SPIN = EulerType.spin
Euler.QUAT = EulerType.quat
Euler.MATRIX = EulerType.matrix
Euler.SGIROT = EulerType.sgirot
Euler.SPIDER = EulerType.spider
Euler.MRC = EulerType.mrc

try:
	a=enumerate([1,2])
except:
	def enumerate(x): return zip(range(len(x)),x)

def displayphase(img):
	"""This will display complex phases of a single image with the phase-origin in the middle"""
	fsp=os.path.basename(tempfile.mktemp())+".mrc"
	if (img.isComplex()) : f=img
	else: f=img.doFFT()
	f.toCorner()
	f.ri2ap()
	for i in range(0,f.xSize()*f.ySize(),2):
		f.setValueAt(i,0,0,f.valueAt(i+1,0,0))
		f.setValueAt(i+1,0,0,0)
	f.ap2ri()

	f.writeImage(fsp)
	os.system("v2 "+fsp)
	os.unlink(fsp)
EMData.displayphase=displayphase

def pseudocomplex(img):
	"""This will create a volume data set from a complex volume where the left half of the image
	represents the phases and the right half represents amplitudes. Note that it is rescaled to
	be cubic in a lossy way."""
	if (type(img)==type("a")) :
		fsp=img
		img=EMData()
		img.readImage(fsp)
		img=img.doFFT()
		img.toCorner()
	if (not img.isComplex()) : return None
	img.ap2ri()
	sig=img.Sigma()
	img.ri2ap()
	ret=EMData()
	ret.setSize(img.xSize()-2,img.ySize(),img.zSize())
	for z in range(ret.zSize()):
		for y in range(ret.ySize()):
			for x in range(ret.xSize()/2):
				ret.setValueAt(ret.xSize()/2-x-1,y,z,img.valueAt(x*2+1,y,z)+math.pi)
				ret.setValueAt(x+ret.xSize()/2,y,z,img.valueAt(x*2,y,z)*math.pi/sig)
	ret.update()
	return ret

def ampcomplex(img):
	"""This will create a volume data set from a complex volume where both halves of the image
	represents amplitudes. Note that it is rescaled to
	be cubic in a lossy way."""
	if (type(img)==type("a")) :
		fsp=img
		img=EMData()
		img.readImage(fsp)
		img=img.doFFT()
		img.toCorner()
	if (not img.isComplex()) : return None
	img.ap2ri()
	sig=img.Sigma()
	img.ri2ap()
	ret=EMData()
	ret.setSize(img.xSize()-2,img.ySize(),img.zSize())
	for z in range(ret.zSize()):
		for y in range(ret.ySize()):
			for x in range(ret.xSize()/2):
				ret.setValueAt(ret.xSize()/2-x-1,y,z,img.valueAt(x*2,y,z))
				ret.setValueAt(x+ret.xSize()/2  ,ret.ySize()-y,z,img.valueAt(x*2,y,z))
	ret.update()
	return ret

def display(lst) :
	"""This is used to display a list of EMData objects"""
	fspbase = os.path.basename(tempfile.mktemp())
	fspimg = fspbase + ".img"
	fsphed = fspbase + ".hed"
	
	for i in lst:
		i.writeImage(fspimg, -1, Image.img)

	os.system("v2 " + fspimg)
	os.unlink(fspimg)
	os.unlink(fsphed)

def triplot(tuplelist):
	"""this will display a list of 2 or 3 tuples in a 'triview'"""
	out=os.popen("triplot -&","w")
	for i in tuplelist:
		if (len(i)==2) : out.write("%f\t%f\n"%(float(i[0]),float(i[1])))
		elif (len(i)>2) : out.write("%f\t%f\t%f\n"%(float(i[0]),float(i[1]),float(i[2])))
	out.close()

def readCTFParm(filename = "ctfparm.txt", return_type="list"):
	"""read in the CTF parameters and return in either a dictionary or a list"""
	ctfparm_dict = { }
	ctfparm_list = [ ]
	for l in open(filename):
		mid, ctfparm = l.strip().split();
		df, b, amp, ac, n1, n2, n3, n4, V, Cs, apix, sffile = ctfparm.split(",")
		ctfparm_dict[mid] = {"defocus":float(df), "B":float(b), "amplitude":amp,  "ac":ac, 
							"n1":float(n1), "n2":float(n2), "n3":float(n3), "n4":float(n4), 
							"V":float(V), "Cs":float(Cs), "apix":float(apix),  "sffile":sffile }
		ctfparm_list.append((mid, float(df), float(b), float(amp), float(ac), float(n1), float(n2), 
							float(n3), float(n4), float(V), float(Cs), float(apix), sffile))
		
	if return_type == "dict": return ctfparm_dict
	else: return ctfparm_list

def readEMANBoxfile(file):	# read one box file in EMAN boxer format
	boxes=[] 
	for l in open(file):
		tokens = l.split()
		nx = int(tokens[2])
		ny = int(tokens[3])
		xc = int(tokens[0])+nx/2
		yc = int(tokens[1])+ny/2
		boxes.append((xc, yc, nx, ny))
	return boxes

def resizeBoxes(new_nx, new_ny):
	boxes = []
	for b in boxes: boxes.append((b[0], b[1], new_nx, new_ny))
	return boxes
	
def scaleBoxes(scale):
	boxes = []
	for b in boxes: boxes.append((int(b[0]*scale), int(b[1]*scale), int(b[2]*scale), int(b[3]*scale)))
	return boxes
	
def writeEMANBoxfile(file, boxes):
	fp = open(file,'w')
	for b in boxes: fp.write("%d\t%d\t%d\t%d\t-3\n" % (b[0]-b[2]/2, b[1]-b[3]/2, b[2], b[3]))
	fp.close()

def parseFocalPairInfo(filename, tolower=0):
	r"""
	the file is a text file with 3 column format: <microscopy date yyyy/mm/dd> <the micrograph/CCD image id> <exposure number>
	most of the time, the file is generated by pydb.py
	"""
	import datetime, re
	
	info={}
	all = []
	for l in open(filename):
		l2 = l.strip()
		if l2[0] == "#": continue
		try:
			day, image, exposure = l2.split()
		except:
			print "file %s is in wrong format" % (filename)
			break
		
		try:
			y, m, d = day.split("/")
		except:
			print "line \"%s\" does not have correct date" % (l2, day)
			break
		
		date = datetime.date(int(y),int(m),int(d))
		if not info.has_key(date): info[date] = {"pairs":{}, "singles_first":[], "singles_other":[]}
		
		if image.startswith("ccd_"): image=image[4:]
		if tolower: image = image.lower()
		
		all.append((date, image, int(exposure)))
		#print all[-1]
	
	re_pattern =  re.compile(r"^(?P<imgid>\d+)(?P<prefix>.*)$")
	def cmp_all(a, b):
		if a[0] != b[0]: 	# first sort according to date
			return cmp(a[0], b[0])
		else:
			try:
				sa = re_pattern.search(a[1][::-1])
				sb = re_pattern.search(b[1][::-1])
				ida = int(sa.group("imgid")[::-1])
				prefixa = sa.group("prefix")[::-1]
				idb = int(sb.group("imgid")[::-1])
				prefixb = sb.group("prefix")[::-1]
				
				if prefixa != prefixb: return cmp(prefixa, prefixb)
				else: return cmp(ida, idb)
			except:
				if a[1] != b[1]: return cmp(a[1], b[1])
				else: return cmp(a[2], b[2])
	
	all.sort(cmp_all)
			
	imgnum = len(all)
	done = [0] * imgnum

	for i in range(imgnum):
		i2 = i+1
		if all[i][2]==1:
			if i2<imgnum and all[i2][2]==2:
				id1 = 0; id2 = 0
				try:
					id1 = int(re_pattern.search(all[i][1][::-1]).group("imgid")[::-1])
					id2 = int(re_pattern.search(all[i2][1][::-1]).group("imgid")[::-1])
				except:
					pass
				if all[i][0] == all[i2][0] and id2==id1+1:
					info[all[i][0]]["pairs"][all[i][1]] = (all[i][1], all[i2][1])
					info[all[i][0]]["pairs"][all[i2][1]] = (all[i][1], all[i2][1])
					done[i] = 1
					done[i2]= 1
					#print "pairs: %s\t%s" % (all[i], all[i2])
		
	for i in range(imgnum):
		if done[i] == 0:
			if all[i][2] == 1: info[all[i][0]]["singles_first"].append(all[i][1])
			else: info[all[i][0]]["singles_other"].append(all[i][1])
			#print "single: %s" % ( str(all[i]) )
			
	return info

def LOGbegin(ARGV):
	if sys.platform == "win32": os.getppid = os.getpid
	out=open(".emanlog","a")
	b=string.split(ARGV[0],'/')
	ARGV[0]=b[len(b)-1]
	out.write("%d\t%d\t1\t%d\t%s\n" % (os.getpid(),time.time(),os.getppid(),string.join(ARGV," ")))
	out.close()
	return os.getpid()

def LOGInfile(pid,file):
	out=open(".emanlog","a")
	out.write("%d\t%d\t%d\t%s\t\n" % (pid,time.time(),10,file))
	out.close()

def LOGReffile(pid,file):
	out=open(".emanlog","a")
	out.write("%d\t%d\t%d\t%s\t\n" % (pid,time.time(),12,file))
	out.close()

def LOGOutfile(pid,file):
	out=open(".emanlog","a")
	out.write("%d\t%d\t%d\t%s\t\n" % (pid,time.time(),11,file))
	out.close()
	
def LOGend():
	out=open(".emanlog","a")
	out.write("%d\t%d\t2\t-\t\n" % (os.getpid(),time.time()))
	out.close()

